version 12
set more off
adopath + ../external/lib/stata/gslab_misc/ado
adopath + ../external/lib/stata/ado
preliminaries

program main 
    ip_visits
end 

program ip_visits
    clear
    foreach year in 07 08 09 10 11 12 {
        append using ../external_links/ip`year'/ipr20`year'
    }
    /*Identifiying ER visits through ResDAC defintion
    https://www.resdac.org/resconnect/articles/144 */
    gen er = rev_cntr=="0981"
    forvalues c=450/459 {
        local code = "0"+"`c'"
        replace er = 1 if rev_cntr=="`code'"
    }
    gen icu = inlist(rev_cntr, "0200", "0201", "0202", "0203", "0204", "0206", "0207", "0208", "0209")

    collapse (max) er icu, by(bene_id clm_id)
    save ../temp/er_icu, replace
    clear
    foreach year in 07 08 09 10 11 12 {
        append using ../external_links/ip`year'/ipc20`year'
    }
    merge 1:1 bene_id clm_id using ../temp/er_icu, assert(3) keep(3) keepusing(er icu) nogen
    scalar all_ip_obs = _N
    isid bene_id clm_id  // obs uniquely identified by bene_id-clm_id
    sort bene_id admsn_dt from_dt thru_dt  // ensures collapse (first) is consistently defined

    count if dschrgdt == .
    scalar mi_dschrgdt = r(N)

    gen miss_dschrgdt = dschrgdt == .
    egen has_missing_dschrgdt = max(miss_dschrgdt), by(bene_id admsn_dt)
    egen last_dschrgdt = max(dschrgdt), by(bene_id admsn_dt)
    gen has_dschrgdt = last_dschrgdt != .
    tab has_dschrgdt if has_missing_dschrgdt == 1  // most bene_id-admsn_dt pairs that have a missing discharge date also have a nonmissing discharge date
    gsort -has_missing_dschrgdt +bene_id +admsn_dt +from_dt
    list bene_id clm_id admsn_dt dschrgdt last_dschrgdt from_dt thru_dt in 1/50
    // replace dschrgdt to be nonmissing dschrgdt for consectuive claims https://www.resdac.org/cms-data/variables/Acute-Inpatient-Stays
    replace dschrgdt = last_dschrgdt if has_missing_dschrgdt == . & dschrgdt == . & last_dschrgdt != .

    count if dschrgdt == .
    scalar still_mi_dschrgdt = r(N)
    drop if dschrgdt == .  // .3% (.003 as a share) of obs have missing discharged date
    table_dschrgdt

    assert !mi(admsn_dt)

    replace year = year(thru_dt) if year(thru_dt) > 2008   // year doesn't exist after 2008 (it's based on thru_dt)
    replace dgnscd1 = icd_dgns_cd1 if year > 2009  // var name changed
    replace ad_dgns = admtg_dgns_cd if year > 2009  // var name changed
    forvalues v = 1/6 {
        replace prcdrcd`v' = icd_prcdr_cd`v' if year > 2009
        gen has_prcdrcd`v' = prcdrcd`v'!=""
    }
    egen num_prcdrs = rowtotal(has_prcdrcd1-has_prcdrcd6)
    drop has_prcdrcd?

    neg_checks

    save_data ../output/ip_procedures_check.dta, key(bene_id clm_id) replace

    replace pmt_amt = pmt_amt + per_diem * util_day  // per Ande's email of Jan 12

    collapse (sum) pmt_amt ded_amt coin_amt blddedam prpayamt num_prcdrs ///
        (first) dgnscd1 ad_dgns (max) er icu, by(bene_id admsn_dt dschrgdt)

    gen num_visits = 1
    gen num_icu_visits = icu
    gen num_ip_er_visits = er
    gen num_ip_days = dschrgdt - admsn_dt + 1

    gen year = year(admsn_dt)
    label var year "Year of Admission"
    save_data ../output/ip_visits_by_year, key(bene_id admsn_dt dschrgdt) replace

    forvalues year = 2007/2012 {
        use ../output/ip_visits_by_year, clear
        keep if year == `year'
        scalar obs_`year' = _N
        save_data ../output/ip_visits_`year', key(bene_id admsn_dt dschrgdt) replace
    }
    table_obs
end

    program table_dschrgdt
        matrix dschrg_drop = (all_ip_obs \ mi_dschrgdt \ still_mi_dschrgdt)
        matrix colnames dschrg_drop  = "Obs"
        matrix rownames dschrg_drop = "All ip 07-12" "Mi dschrgdt" "Still mi dschrgdt"
        matrix list dschrg_drop
        matrix_to_txt, matrix(dschrg_drop) saving(../output/tables_counts.txt) ///
            title(<tab:dschrg_drop>) replace
    end

    program neg_checks
        gen neg_pmt = pmt_amt < 0
        gen bene_pay = ded_amt + coin_amt + blddedam
        gen neg_from_deductible = (bene_pay > 0) & (neg_pmt == 1)
        gen final_check = bene_pay + pmt_amt
        gen problem = (final_check <= 0) & (neg_pmt == 1)
        forvalues year = 2007/2012 {
            preserve
            keep if year == `year'
            scalar tot_clms_`year' = _N
            count if neg_pmt == 1
            scalar all_neg_`year' = r(N)
            count if neg_from_deductible == 1
            scalar neg_explained_`year' = r(N)
            count if problem == 1
            scalar still_problem_`year' = r(N)
            neg_table `year'
            restore
        }
        replace pmt_amt = 0 if pmt_amt < 0
        drop neg_pmt bene_pay neg_from_deductible final_check problem
    end

        program neg_table
            args year
            matrix neg_claims = (tot_clms_`year' \ all_neg_`year' \ ///
            neg_explained_`year' \ still_problem_`year')
            matrix colnames neg_claims = "# Claims"
            matrix rownames neg_claims = "Tot claims `year'" "All neg claims" ///
                "Explained neg" "Non-explained neg"
            matrix list neg_claims
            matrix_to_txt, matrix(neg_claims) saving(../output/tables_counts.txt) ///
                title(<tab:neg_claims_`year'>) append
        end

    program table_obs
        matrix obs_mat = (obs_2007 \ obs_2008 \ obs_2009 \ obs_2010 \ obs_2011 \ obs_2012)
        matrix colnames obs_mat = "Obs"
        matrix rownames obs_mat = "2007" "2008" "2009" "2010" "2011" "2012"
        matrix list obs_mat
        matrix_to_txt, matrix(obs_mat) saving(../output/tables_counts.txt) ///
            title(<tab:yearly_obs>) append
    end

*EXECUTE
main
