/***************************************************************************************************
 *
 * BUILD.DO
 *
 * This code reads in the denominator from csv files, cleans entries and generates variables of 
 * interest. Datasets are saved that include the denominator for the study years (2008-2010), the 
 * list of beneficiaries from 2007-2011, and the list of beneficiaries from 2008-2010.
 *
 **************************************************************************************************/

version 12
set more off
adopath + ../external/lib/stata/gslab_misc/ado
preliminaries

program main
    append_denom
    dual_month
end

program append_denom
    foreach y in 07 08 09 10 11 {
        insheet using ../external/denom/denom2020`y'_cnfm.csv, clear
        tempfile d`y'
        save `d`y''
    }
    clear
    foreach y in 07 08 09 10 11 {
        append using `d`y''
    }
    scalar raw_obs_total = _N
    duplicates drop bene_id year, force  // XX 582 duplicates - check this
    save ../temp/denom_append, replace

    recode_dates
    recode_discrepancies
    hmo
    dual
    part_a_b
    age_jan1_08
    keep bene_id year sex race sdo? dual* age_jan1_08 jan1_08_65plus ///
        zipcode county_de state_de orec crec esrd_ind pta_* ptb_* hmo_*

    save_data ../output_local/denom.dta, key(bene_id year) replace ///
        log(../output/data_file_manifest.log)
    describe using ../output_local/denom.dta

    keep bene_id sex race sdo? orec //Not including time-varying variables
    duplicates drop
    scalar full_obs = _N

    save_data ../output_local/denom_benes.dta, key(bene_id) replace ///
        log(../output/data_file_manifest.log)
    describe using ../output_local/denom_benes.dta
end

    program recode_dates
        ren sdo? sdo?_s
        gen sdob = date(sdob_s,"DMY")
        gen sdod = date(sdod_s,"DMY")
        format %td sdob sdod
        drop sdo?_s
    end

    program recode_discrepancies
        ren sdob sdob_y
        egen sdob = mode(sdob), max by(bene_id)
        label var sdod "modal date of birth within a bene"
        ren sdod sdod_y
        egen sdod = mode(sdod), max by(bene_id)
        label var sdod "modal (nonmissing) date of death within a bene"
        replace race = race!=1
        ren race race_y
        egen race = mode(race), max by(bene_id)
        label var race "I: takes value 0 if modal race within a bene is white"
        ren sex sex_y
        egen sex = mode(sex_y), max by(bene_id)
        label var sex "I: takes value 2 if modal sex within a bene is female"
        ren orec orec_y
        egen orec = mode(orec), max by(bene_id)
        label var orec "modal original reason for entitlement"
    end
 
    program hmo
        forvalues m=1/12 {
            // www.resdac.org/cms-data/variables/HMO-Coverage-Count
            gen hmo_`m' = !inlist(hmoind`m',"0","4")
        }
    end

    program dual
        forvalues m=1/12 {
            gen dual_`m' = inlist(buyin`m',"A","B","C")
        }
    end

    program part_a_b
        forval month = 1/12 {
            gen pta_`month' = (buyin`month' == "1" | buyin`month' == "3" | ///
                buyin`month' == "A" | buyin`month' == "C")
            gen ptb_`month' = (buyin`month' == "2" | buyin`month' == "3" | ///
                buyin`month' == "B" | buyin`month' == "C")
        }
    end

    program age_jan1_08
        gen age_jan1_08 = (td(1jan2008)-sdob)/365.25
        gen jan1_08_65plus = age_jan1_08 >= 65 & !mi(age_jan1_08)
    end

program dual_month
    use ../output_local/denom.dta, clear
    keep bene_id year dual_*
    reshape long dual_, i(bene_id year) j(month)
    rename dual_ month_dual
    egen year_dual = max(month_dual), by (bene_id year)
    egen ever_dual = max(year_dual), by (bene_id)
    save_data ../output_local/dual_benes.dta, key(bene_id year month) replace ///
        log(../output/data_file_manifest.log)
    describe using ../output_local/dual_benes.dta
end

* EXECUTE
main
