version 12
set more off
adopath + ../external/lib/stata/gslab_misc/ado
preliminaries

program main
    foreach year in 2008 2011 {
        define_t0 `year'
        drop_hmo `year'
        drop_partA_partB `year'
        define_mortality `year'
        merge_hrrs `year'
        gen_age_at_t0 `year'
        mcaid `year'
        save_sample `year'
    }
end

program define_t0
    args year
    use ../external_links/denom/denom.dta, clear
    gen t0 = date("1JAN`year'", "DMY")
    gen event_month = month(t0)
    save ../temp/denom_`year', replace
end

program drop_hmo
    args year
    use ../temp/denom_`year', clear
    gen pre_hmo_months = 0
    gen post_hmo_months = 0
    forvalues month=1/12 {
        gen pre_hmo_`month'_tmp = hmo_`month' if year==`year'-1
        egen pre_hmo_`month' = max(pre_hmo_`month'_tmp), by(bene_id)
        gen post_hmo_`month'_tmp = hmo_`month' if year==`year'+1
        egen post_hmo_`month' = max(post_hmo_`month'_tmp), by(bene_id)
        replace post_hmo_`month' = 0 if post_hmo_`month'==.
        drop pre_hmo_*_tmp post_hmo_*_tmp
        replace pre_hmo_months = pre_hmo_months + hmo_`month' if event_month > `month'
        replace pre_hmo_months = pre_hmo_months + pre_hmo_`month' if event_month <= `month'
        replace post_hmo_months = post_hmo_months + post_hmo_`month' if event_month > `month'
        replace post_hmo_months = post_hmo_months + hmo_`month' if event_month <= `month'
    }
    gen pre_hmo_no_months_tmp = pre_hmo_months==0 if year==`year'
    egen pre_hmo_no_months = max(pre_hmo_no_months_tmp), by(bene_id) 
    gen post_hmo_no_months_tmp = post_hmo_months==0 if year==`year'
    egen post_hmo_no_months = max(post_hmo_no_months_tmp), by(bene_id) 
    unique bene_id
    display "Unique IDs: " r(N)
    count if (!pre_hmo_no_months | !post_hmo_no_months) & year==`year'
    display "Patients in HMO in at least one month in pre- or post-year: " r(N)
    drop if !pre_hmo_no_months | !post_hmo_no_months
    save ../temp/building_`year', replace
end

program drop_partA_partB
    args year
    use ../temp/building_`year', clear
    gen partA_months = 0
    gen partB_months = 0
    forvalue month = 1/12 {
        gen prev_yr_pta_`month'_tmp = pta_`month' if year==`year'-1
        gen prev_yr_ptb_`month'_tmp = ptb_`month' if year==`year'-1
        egen prev_yr_pta_`month' = max(prev_yr_pta_`month'_tmp), by(bene_id)
        egen prev_yr_ptb_`month' = max(prev_yr_ptb_`month'_tmp), by(bene_id)
        drop prev_yr_*_tmp
        replace partA_months = partA_months + pta_`month' if event_month > `month' 
        replace partA_months = partA_months + prev_yr_pta_`month' if event_month <= `month' 
        replace partB_months = partB_months + ptb_`month' if event_month > `month'
        replace partB_months = partB_months + prev_yr_ptb_`month' if event_month <= `month' 
    }
    gen partA_all_months_tmp = partA_months==12 if year==`year'
    gen partB_all_months_tmp = partB_months==12 if year==`year'
    egen partA_all_months = max(partA_all_months_tmp), by(bene_id) 
    egen partB_all_months = max(partB_all_months_tmp), by(bene_id) 
    unique bene_id
    display "Unique IDs: " r(N)
    count if (!partA_all_months | !partB_all_months) & year==`year'
    display "Patients without Part A and B in current month and 12 prior months: " r(N)
    drop if !partA_all_months | !partB_all_months
    save ../temp/base_`year', replace
end

program define_mortality
    args year
    use ../temp/base_`year', clear
    gen days_t0_to_death_tmp = sdod-t0 if year==`year'
    egen days_t0_to_death = max(days_t0_to_death_tmp), by(bene_id)
    drop days_t0_to_death_tmp
    drop if days_t0_to_death<0
    foreach d in 30 60 90 180 270 365 {
        gen mort`d' = days_t0_to_death<`d'
    }
    save ../temp/base_`year', replace
end

program merge_hrrs
	args year
    use ../temp/base_`year', clear
    format %05.0f zipcode
	tostring zipcode, u replace
	merge m:1 zipcode using ../external/hrrxw/ziphrrxwalk, ///
      assert(1 2 3) keep(3) keepusing(hrrnum) nogen
      * in 2 if nobody in that zipcode, in 1 if zipcode not in xwalk
      * only keep if has hrrnum
    save ../temp/base_`year', replace
end

program gen_age_at_t0
    args year
    use ../temp/base_`year', clear
    gen age_at_t0 = year(t0) - year(sdob)
    replace age_at_t0 = year(t0) - year(sdob) - 1 if month(t0)<month(sdob)
    replace age_at_t0 = year(t0) - year(sdob) - 1 if day(t0)<day(sdob) & month(t0)==month(sdob)
    rename age_at_t0 age
    save ../temp/base_`year', replace
end

program mcaid
    args year
    use ../temp/base_`year', clear
    gen mcaid_months = 0
    forvalues month = 1/12 {
        gen prev_yr_mcaid_`month'_tmp = dual_`month' if year == `year' - 1 
        egen prev_yr_mcaid_`month' = max(prev_yr_mcaid_`month'_tmp), by(bene_id)
        drop prev_yr_mcaid_*_tmp
        replace mcaid_months = mcaid_months + dual_`month' if event_month > `month' 
        replace mcaid_months = mcaid_months + prev_yr_mcaid_`month' if event_month <= `month' 
    }
    gen mcaid_any_month_tmp = mcaid_months>0 if year==`year' 
    egen mcaid = max(mcaid_any_month_tmp), by(bene_id) 
    save ../temp/base_`year', replace  
end

program save_sample
    args year
    use ../temp/base_`year'.dta, clear
    replace sex = sex==2
    keep if year==`year'
    keep bene_id t0 age days_t0_to_death sdob sdod ///
      mort* race sex mcaid state_de county_de zipcode hrrnum year
    order bene_id t0 age days_t0_to_death sdob sdod ///
      mort* race sex mcaid state_de county_de zipcode hrrnum year
    save_data ../output_local/1jan`year'.dta, key(bene_id) replace ///
      log(../output/data_file_manifest.log)
end
 
* EXECUTE
main
