#! /usr/bin/env python

import os
import re
import shutil 
import subprocess

import messages as messages
import metadata as metadata
from exceptionclasses import CustomError, CritError, SyntaxError, LogicError

######################################################
# System Directive
######################################################

class SystemDirective(object):


    def __new__(cls, raw, LOGFILE, last_dir, last_rev):
        if raw.split('\t',1)[0]!='COPY':
            return super(SystemDirective,cls).__new__(SvnExportDirective)
        else:
            return super(SystemDirective,cls).__new__(CopyDirective)

    def __init__(self, raw, LOGFILE, last_dir, last_rev):
        self.logfile = LOGFILE
        # Print raw input to logfile
        print >> self.logfile, messages.note_input % raw
        # Split columns
        self.raw = raw.replace('    ','\t')
        list = ((self.raw.split('\t')+[None])[:5])
        for i in xrange(len(list)):
            list[i] = list[i].strip()
        self.rev,self.dir,self.file,self.outdir,self.outfile = list
        # Load previous rev/dir if empty 
        if self.dir=='' and self.rev=='':
            self.dir = last_dir
            self.rev = last_rev
        if not re.search('/$', self.dir):
            self.dir = self.dir + '/'
        # Flag if wildcard list needs to be compiled in subclass
        self.flag_list = []
        if re.search('\*.+',self.file) or re.search('^.+\*',self.file) or (re.search('\*',self.outfile)):
            self.flag_list = 1

    # Error Check
    def error_check(self):
        # Check that rev/dir are well-defined
        if self.dir=='' and self.rev=='':
            if self.last_rev=='' or self.last_dir=='':
                raise LogicError(messages.logic_error_firstrevdir)

        # Check syntax
        if self.file=='' or self.outfile=='' or self.outdir=='':
            raise SyntaxError(messages.syn_error_noname)
        if not re.search('\*',self.file) and re.search('\*',self.outfile):
            raise SyntaxError(messages.syn_error_wildfilename1.replace('\n',''))
        if re.search('\*',self.outfile) and not re.search('\*$',self.outfile):
            raise SyntaxError(messages.syn_error_wildoutfile.replace('\n',''))
            

        # Check wildcard directory non-empty
        if self.flag_list and not self.LIST:
            raise LogicError(messages.crit_error_emptylist)

    # Clean
    def clean(self, externals_dir):
        self.clean_syntax()
        self.clean_logic(externals_dir)

    def clean_syntax(self):
        # Clean slashes
        if self.outdir=='.' or self.outdir=='.\\':
            self.outdir = './'
        self.outdir = re.sub('^\.\\.*\\$', '^/.*/$', self.outdir)
        if not re.search('/$',self.outdir):
            self.outdir = self.outdir+'/'

    def clean_logic(self, externals_dir):
        # Replace '.' in externals.txt with inputted externals directory location
        if self.outdir=='./':
            self.outdir = externals_dir
        else:
            self.outdir = externals_dir+self.outdir

        # Prepare prefix
        self.outprefix = ''
        if re.search('\*',self.outfile):
            self.outprefix = self.outfile[0:-1]

        # If outfile is left as a double quote, keep the same file name
        if self.outfile=='""""':
            self.outfile = self.file

        # Set file/outfile to empty if entire directory is operated on
        if self.file=='*':
            self.file = ''
            self.outfile = ''

    # Issue System Command
    def issue_sys_command(self, quiet):
        if not os.path.exists(self.outdir):
            os.makedirs(self.outdir)

        if self.flag_list:
            print >> self.logfile, messages.note_array % self.LIST
            for element in self.LIST:
                #Add prefix to outfile name (if none, then no change)
                self.outfile = self.outprefix+element
                self.file = element
                self.command(quiet)
        else:
            self.command(quiet)

    def command(self, quiet):
        # Define individually for each subclass
        pass

######################################################
# SvnExport Directive
######################################################

class SvnExportDirective(SystemDirective):


    def __init__(self, raw, LOGFILE, last_dir, last_rev):
        SystemDirective.__init__(self, raw, LOGFILE, last_dir, last_rev)
        # Fill in placemarkers
        self.dir = re.sub('\%svn\%',metadata.file_loc['svnroot'],self.dir)
        self.dir = re.sub('\%svnbranch\%',metadata.file_loc['svnbranch'],self.dir)
        # Make wildcard list
        if self.flag_list:
            svnlist = 'svn list \"%s@%s\"' % (self.dir,self.rev)
            LISTSVN = os.popen(svnlist).readlines()
            wildfirst,wildlast = self.file.split('*')
            self.LIST = []
            for element in LISTSVN:
                if re.match('%s.*%s$' % (wildfirst,wildlast),element):
                    self.LIST.append(element.rstrip('\n'))

    # Error Check
    def error_check(self):
        if not re.match('\d+$',self.rev) and self.rev!='':
            raise SyntaxError(messages.syn_error_revnum % (self.rev,self.dir,self.file))
        if self.dir=='' and self.rev!='' or self.dir!='' and self.rev=='':
            raise LogicError(messages.logic_error_revdir.replace('\n','') % (self.dir,self.file,
                            self.rev,self.dir))

 
        SystemDirective.error_check(self)

    # Define SvnExport Command    
    def command(self, quiet):
        # Following command contains try/except clause
        if quiet:
            subprocess.check_call(metadata.commands['svnexport'] % (self.rev, self.dir, self.file,
                                self.rev, self.outdir, self.outfile), shell=True, stdout=open(os.devnull, 'w'), stderr=open(os.devnull, 'w'))
        else:
            subprocess.check_call(metadata.commands['svnexport'] % (self.rev, self.dir, self.file,
                                self.rev, self.outdir, self.outfile), shell=True)        
        print >> self.logfile, messages.success_svn % (self.dir, self.file, self.rev,
                                                       self.outdir, self.outfile)

######################################################
# Copy Directive
######################################################

class CopyDirective(SystemDirective):


    def __init__(self, raw, LOGFILE, last_dir, last_rev):
        SystemDirective.__init__(self, raw, LOGFILE, last_dir, last_rev)
        # Deal with any directory itemwise
        if re.search('\*',self.file):
            self.flag_list = 1
        # Fill in placemarkers, and dir if empty
        if self.dir=='':
            self.dir = self.last_dir
        for key in metadata.file_loc:
            self.dir = re.sub('\%%%s\%%' % key,metadata.file_loc[key],self.dir)
        # Ensure legal slashes
        self.dir = self.dir.replace('/','\\')
        # Make wildcard list
        if self.flag_list:
            self.LIST = os.listdir(self.dir)

    # Error Check
    def check_revdir(self):
        if self.dir=='' and self.last_dir=='':
            raise LogicError(messages.logic_error_firstrevdir)


        SystemDirective.check_revdir(self)

    # Define Copy Command        
    def command(self, quiet):
        try:
            shutil.copy('%s%s' % (self.dir, self.file),'%s%s' % (self.outdir, self.outfile))
            print >> self.logfile, messages.success_copy  % (self.dir, self.file, 
                                                             self.outdir, self.outfile)
        except:
            raise CritError(messages.crit_error_copyincomplete % (self.dir,self.file))