#! /usr/bin/env python

import unittest, sys, os, shutil, contextlib
sys.path.append('..')
from py.make_log import start_make_logging
from py.dir_mod import clear_output_dirs
from py.run_program import run_sas
from nostderrout import nostderrout

    
class testRunSas(unittest.TestCase):

    def setUp(self):
        makelog_file = '../output/make.log'
        output_dir = '../output/'
        with nostderrout():
            clear_output_dirs(output_dir, '')
            start_make_logging(makelog_file)

    def test_default_log(self):
        with nostderrout():
            run_sas(program = './input/sas_test_script.sas')
        logfile_data = open('../output/make.log', 'rU').read()
        self.assertIn('NOTE: The SAS System used:', logfile_data)
        self.assertNotIn('Errors printed on page', logfile_data)
        self.assertTrue(os.path.isfile('out_data.csv'))
        self.assertTrue(os.path.isfile('../output/sas_test_script.lst'))   

    def test_no_lst(self):
        with nostderrout():
            run_sas(program = './input/sas_test_script_nolst.sas')
        logfile_data = open('../output/make.log', 'rU').read()
        self.assertIn('NOTE: The SAS System used:', logfile_data)
        self.assertNotIn('Errors printed on page', logfile_data)
        self.assertTrue(os.path.isfile('out_data.csv'))
        self.assertFalse(os.path.isfile('../output/sas_test_script.lst'))         
       
    def test_custom_log(self):
        os.remove('../output/make.log')
        makelog_file = '../output/custom_make.log'
        output_dir = '../output/'
        with nostderrout():
            clear_output_dirs(output_dir, '')
            start_make_logging(makelog_file)
            run_sas(program = './input/sas_test_script.sas', makelog = '../output/custom_make.log')
        logfile_data = open('../output/custom_make.log', 'rU').read()
        self.assertIn('NOTE: The SAS System used:', logfile_data)
        self.assertNotIn('Errors printed on page', logfile_data)
        self.assertTrue(os.path.isfile('out_data.csv'))
        self.assertTrue(os.path.isfile('../output/sas_test_script.lst'))         
        
    def test_independent_log(self):
        with nostderrout():
            run_sas(program = './input/sas_test_script.sas', log = '../output/sas.log')
        makelog_data = open('../output/make.log', 'rU').read()
        self.assertIn('NOTE: The SAS System used:', makelog_data)
        self.assertNotIn('Errors printed on page', makelog_data)
        self.assertTrue(os.path.isfile('../output/sas.log'))
        saslog_data = open('../output/sas.log', 'rU').read()
        self.assertIn('NOTE: The SAS System used:', saslog_data)
        self.assertNotIn('Errors printed on page', saslog_data)
        self.assertTrue(os.path.isfile('out_data.csv'))
        self.assertTrue(os.path.isfile('../output/sas_test_script.lst'))        

    def test_no_extension(self):
        with nostderrout():
            run_sas(program = './input/sas_test_script')
        logfile_data = open('../output/make.log', 'rU').read()
        self.assertIn('NOTE: The SAS System used:', logfile_data)
        self.assertNotIn('Errors printed on page', logfile_data)
        self.assertTrue(os.path.isfile('out_data.csv'))
        self.assertTrue(os.path.isfile('../output/sas_test_script.lst'))         
        
    def test_executable(self):
        with nostderrout():
            run_sas(program = './input/sas_test_script.sas', executable = 'sas') 
        logfile_data = open('../output/make.log', 'rU').read()
        self.assertIn('NOTE: The SAS System used:', logfile_data)
        self.assertNotIn('Errors printed on page', logfile_data)
        self.assertTrue(os.path.isfile('out_data.csv'))
        self.assertTrue(os.path.isfile('../output/sas_test_script.lst'))        

    def test_lst(self):
        with nostderrout():
            run_sas(program = './input/sas_test_script.sas', lst = './')
        logfile_data = open('../output/make.log', 'rU').read()
        self.assertIn('NOTE: The SAS System used:', logfile_data)
        self.assertNotIn('Errors printed on page', logfile_data)
        self.assertTrue(os.path.isfile('out_data.csv'))
        self.assertTrue(os.path.isfile('sas_test_script.lst'))
        self.assertFalse(os.path.isfile('../output/sas_test_script.lst'))        
        
    def test_bad_executable(self):
        with nostderrout():    
            run_sas(program = './input/sas_test_script.sas', executable = 'nonexistent_sas_executable')
        logfile_data = open('../output/make.log', 'rU').read()
        self.assertIn('\'nonexistent_sas_executable\' is not recognized as an internal or external command', logfile_data)
    
    def test_no_program(self):
        with nostderrout():
            run_sas(program = './input/nonexistent_sas_script.sas')
        logfile_data = open('../output/make.log', 'rU').readlines()
        end_line_number = len(logfile_data)-1
        self.assertTrue(logfile_data[end_line_number].startswith('CritError:'))
    
    def test_option(self):
        with nostderrout():
            run_sas(program = './input/sas_test_script.sas', option = '-altlog \'alt.log \'')
        logfile_data = open('../output/make.log', 'rU').read()
        self.assertIn('NOTE: The SAS System used:', logfile_data)
        self.assertNotIn('Errors printed on page', logfile_data)
        self.assertTrue(os.path.isfile('out_data.csv'))
        self.assertTrue(os.path.isfile('../output/sas_test_script.lst'))
        self.assertTrue(os.path.isfile('alt.log'))
    
    def test_two_option(self):
        with nostderrout():
            run_sas(program = './input/sas_test_script.sas', option = '-altlog \'alt.log \' -rtraceloc \'rtrace.log\'')
        logfile_data = open('../output/make.log', 'rU').read()
        self.assertIn('NOTE: The SAS System used:', logfile_data)
        self.assertNotIn('Errors printed on page', logfile_data)
        self.assertTrue(os.path.isfile('out_data.csv'))
        self.assertTrue(os.path.isfile('../output/sas_test_script.lst'))
        self.assertTrue(os.path.isfile('alt.log'))
        self.assertTrue(os.path.isfile('rtrace.log'))
    
    def test_sas_error(self):
        with nostderrout():
            run_sas(program = './input/sas_test_script_error.sas')
        logfile_data = open('../output/make.log', 'rU').read()
        self.assertIn('executed with errors', logfile_data)
    
    def test_change_dir(self):        
        with nostderrout():
            run_sas(program = './input/sas_test_script.sas', changedir = True)
        logfile_data = open('../output/make.log', 'rU').read()
        self.assertIn('ERROR: Physical file does not exist', logfile_data)
    
    def tearDown(self):
        if os.path.isdir('../output/'):
            shutil.rmtree('../output/')
        if os.path.isfile('out_data.csv'):
            os.remove('out_data.csv')
        if os.path.isfile('sas_test_script.lst'):
            os.remove('sas_test_script.lst')                
        if os.path.isfile('alt.log'):
            os.remove('alt.log') 
        if os.path.isfile('rtrace.log'):
            os.remove('rtrace.log') 
                                
if __name__ == '__main__':
    os.getcwd()
    unittest.main()